@@ -1,5 +1,5 @@
 """
-This file is part of pycos project. See https://pycos.sourceforge.io for details.
+This file is part of pycos; see https://pycos.org for details.
 
 This module provides API for asynchronous file and pipe processing.  They work
 with Windows, Linux, OS X and likely other UNIX variants. Note that regular
@@ -26,7 +26,7 @@ from pycos import _AsyncPoller, Pycos, Task
 __author__ = "Giridhar Pemmasani (pgiri@yahoo.com)"
 __copyright__ = "Copyright (c) 2014 Giridhar Pemmasani"
 __license__ = "Apache 2.0"
-__url__ = "https://pycos.sourceforge.io"
+__url__ = "https://pycos.org"
 
 __all__ = ['AsyncFile', 'AsyncPipe']
 
