@@ -398,12 +398,20 @@ def _dispycos_server_proc():
                 continue
             _dispycos_reply_task.send(-1)
 
+    exc = KeyboardInterrupt()
+    for _dispycos_var in _dispycos_job_tasks:
+        _dispycos_var.throw(exc)
+    for _dispycos_var in range(10):
+        if (yield _dispycos_jobs_done.wait(timeout=0.5)):
+            break
+        logger.debug('dispycos server "%s": waiting for %s tasks from %s to finish',
+                     _dispycos_name, len(_dispycos_job_tasks), _dispycos_scheduler_task.location)
     # kill any pending jobs
+    logger.debug('dispycos server "%s": terminating %s tasks',
+                 _dispycos_name, len(_dispycos_job_tasks))
     for _dispycos_var in _dispycos_job_tasks:
         _dispycos_var.terminate()
     _dispycos_job_tasks.clear()
-    logger.debug('dispycos server "%s": %s tasks terminated',
-                 _dispycos_name, len(_dispycos_job_tasks))
     _dispycos_msg = {'status': Scheduler.ServerDisconnected, 'location': _dispycos_task.location,
                      'auth': _dispycos_auth, 'pid': _dispycos_config['pid']}
     _dispycos_scheduler_task.send(_dispycos_msg)
@@ -443,7 +451,7 @@ def _dispycos_server_process(_dispycos_mp_queue, _dispycos_config):
     _dispycos_auth = _dispycos_config['auth']
     _dispycos_queue, _dispycos_mp_queue = _dispycos_mp_queue, None
     config = {}
-    for _dispycos_var in ['udp_port', 'tcp_port', 'host', 'ext_ip_addr', 'name', 'discover_peers',
+    for _dispycos_var in ['udp_port', 'tcp_port', 'host', 'ext_host', 'name', 'discover_peers',
                           'secret', 'certfile', 'keyfile', 'dest_path', 'max_file_size',
                           'ipv4_udp_multicast']:
         config[_dispycos_var] = _dispycos_config.pop(_dispycos_var, None)
@@ -585,18 +593,7 @@ def _dispycos_spawn(_dispycos_node_q, _dispycos_spawn_q, _dispycos_config, _disp
         pycos.logger.setLevel(pycos.logger.INFO)
     pycos_scheduler = pycos.Pycos.instance()
 
-    class Struct(object):
-
-        def __init__(self, **kwargs):
-            self.__dict__.update(kwargs)
-
-        def __setattr__(self, name, value):
-            if hasattr(self, name):
-                self.__dict__[name] = value
-            else:
-                raise AttributeError('Invalid attribute "%s"' % name)
-
-    servers = [Struct(sid=sid, port=port, busy_time=busy_time, proc=None, status=None, pid=0)
+    servers = [pycos.Struct(sid=sid, port=port, busy_time=busy_time, proc=None, status=None, pid=0)
                for sid, port, busy_time in _dispycos_server_params]
     mp_q = multiprocessing.Queue()
     lock = threading.Lock()
@@ -674,11 +671,17 @@ def _dispycos_spawn(_dispycos_node_q, _dispycos_spawn_q, _dispycos_config, _disp
         # for server in children:
         #     server.pid = 0
         for server in children:
-            for j in range(10):
+            for j in range(30):
                 proc = server.proc
                 try:
                     if not proc or not proc.is_alive():
                         break
+                    if j == 0:
+                        if os.name == 'nt':
+                            signum = signal.SIGBREAK
+                        else:
+                            signum = signal.SIGINT
+                        os.kill(proc.pid, signum)
                     proc.join(0.2)
                 except ValueError:
                     break
@@ -1048,18 +1051,7 @@ def _dispycos_node():
 
     _dispycos_config['discover_peers'] = False
 
-    class Struct(object):
-
-        def __init__(self, **kwargs):
-            self.__dict__.update(kwargs)
-
-        def __setattr__(self, name, value):
-            if hasattr(self, name):
-                self.__dict__[name] = value
-            else:
-                raise AttributeError('Invalid attribute "%s"' % name)
-
-    service_times = Struct(start=None, stop=None, end=None)
+    service_times = pycos.Struct(start=None, stop=None, end=None)
     # time at start of day
     _dispycos_var = time.localtime()
     _dispycos_var = (int(time.time()) - (_dispycos_var.tm_hour * 3600) -
@@ -1197,14 +1189,14 @@ def _dispycos_node():
 
     for _dispycos_id in range(0, num_cpus + 1):
         _dispycos_var = os.path.join(dispycos_path, '..', 'server-%d.pkl' % _dispycos_id)
-        node_servers[_dispycos_id] = Struct(
+        node_servers[_dispycos_id] = pycos.Struct(
             id=_dispycos_id, pid=0, task=None, name='%s_server-%s' % (node_name, _dispycos_id),
             port=node_ports[_dispycos_id], restart=False, pid_file=_dispycos_var, done=pycos.Event(),
             busy_time=multiprocessing.RawValue('L', 0)
         )
     node_servers[0].name = None
 
-    client_info = Struct(auth=None, scheduler=None, client_location=None, cpus_reserved=0,
+    client_info = pycos.Struct(auth=None, scheduler=None, client_location=None, cpus_reserved=0,
                          spawn_mpproc=None, interval=_dispycos_config['max_pulse_interval'],
                          zombie_period=0, restart_servers=False, served=0, node_q=None, spawn_q=None)
 
@@ -1245,7 +1237,7 @@ def _dispycos_node():
         dispycos_ppid = 1
 
     server_config = {}
-    for _dispycos_var in ['udp_port', 'tcp_port', 'host', 'ext_ip_addr', 'name',
+    for _dispycos_var in ['udp_port', 'tcp_port', 'host', 'ext_host', 'name',
                           'discover_peers', 'secret', 'certfile', 'keyfile', 'dest_path',
                           'max_file_size', 'ipv4_udp_multicast']:
         server_config[_dispycos_var] = _dispycos_config.get(_dispycos_var, None)
@@ -1288,7 +1280,7 @@ def _dispycos_node():
                                            'pid': pid, 'auth': client_info.auth})) == 1:
                 if not terminate:
                     return
-                yield server.done.wait(timeout=7)
+                yield server.done.wait(timeout=10)
                 if server.done.is_set():
                     return
 
@@ -1320,7 +1312,7 @@ def _dispycos_node():
                 client_info.spawn_mpproc = None
                 return 0
             client_info.spawn_q.put({'msg': 'quit', 'auth': client_info.auth})
-            for j in range(5):
+            for j in range(10):
                 try:
                     msg = client_info.node_q.get(True, 2)
                 except Exception:
@@ -2062,10 +2054,11 @@ if __name__ == '__main__':
     parser.add_argument('-c', '--cpus', dest='cpus', type=int, default=0,
                         help='number of CPUs/dispycos instances to run; '
                         'if negative, that many CPUs are not used')
-    parser.add_argument('-i', '--ip_addr', dest='host', action='append', default=[],
-                        help='IP address or host name of this node')
-    parser.add_argument('--ext_ip_addr', dest='ext_ip_addr', action='append', default=[],
-                        help='External IP address to use (needed in case of NAT firewall/gateway)')
+    parser.add_argument('-i', '--host', dest='host', action='append', default=[],
+                        help='host name or IP address of this node')
+    parser.add_argument('--ext_host', dest='ext_host', action='append', default=[],
+                        help='host name or IP address of router to use '
+                        '(needed in case of NAT firewall/gateway)')
     parser.add_argument('--scheduler_port', dest='scheduler_port', type=str,
                         default=eval(pycos.config.DispycosSchedulerPort),
                         help='port number for dispycos scheduler')
