@@ -20,6 +20,10 @@ import ssl
 import struct
 import re
 import platform
+try:
+    import ipaddress
+except ImportError:
+    ipaddress = None
 try:
     import netifaces
 except ImportError:
@@ -84,9 +88,8 @@ class Pycos(pycos.Pycos, metaclass=Singleton):
     'name' is used in locating peers. They must be unique. If 'name' is not
     given, it is set to string 'host:tcp_port'.
 
-    'ext_ip_addr' is either the IP address or list of IP addresses of NAT
-    firewall/gateway if pycos is behind that firewall/gateway. If it is a list,
-    each element must correspond to element of 'host' list.
+    'ext_host' is (list of) host name(s) / IP address(es) of NAT
+    firewall/gateway if pycos is behind that firewall/gateway.
 
     If 'discover_peers' is True (default), this host broadcasts message to
     detect other peers. If it is False, message is not broadcasted.
@@ -107,7 +110,7 @@ class Pycos(pycos.Pycos, metaclass=Singleton):
     _pycos = None
     _pycos_class = pycos.Pycos
 
-    def __init__(self, udp_port=pycos.config.NetPort, tcp_port=None, host=None, ext_ip_addr=None,
+    def __init__(self, udp_port=pycos.config.NetPort, tcp_port=None, host=None, ext_host=None,
                  socket_family=None, ipv4_udp_multicast=False, name=None, discover_peers=True,
                  secret='', certfile=None, keyfile=None, notifier=None,
                  dest_path=None, max_file_size=None):
@@ -148,10 +151,10 @@ class Pycos(pycos.Pycos, metaclass=Singleton):
                 hosts = [None]
         else:
             hosts = [host]
-        if isinstance(ext_ip_addr, list):
-            ext_ip_addrs = ext_ip_addr
+        if isinstance(ext_host, list):
+            ext_hosts = ext_host
         else:
-            ext_ip_addrs = [ext_ip_addr]
+            ext_hosts = [ext_host]
 
         if not name:
             name = socket.gethostname()
@@ -159,12 +162,12 @@ class Pycos(pycos.Pycos, metaclass=Singleton):
         location = None
         for i in range(len(hosts)):
             host = hosts[i]
-            if len(ext_ip_addrs) > i and ext_ip_addrs[i]:
-                ext_ip_addr = Pycos.host_ipaddr(ext_ip_addrs[i])
-                if not ext_ip_addr:
-                    logger.warning('invalid ext_ip_addr "%s" ignored', ext_ip_addrs[i])
+            if len(ext_hosts) > i and ext_hosts[i]:
+                ext_host = Pycos.host_ipaddr(ext_hosts[i])
+                if not ext_host:
+                    logger.warning('invalid ext_host "%s" ignored', ext_hosts[i])
             else:
-                ext_ip_addr = None
+                ext_host = None
             addrinfo = Pycos.host_addrinfo(host=host, socket_family=socket_family,
                                            ipv4_multicast=self.ipv4_udp_multicast)
             if not addrinfo:
@@ -201,8 +204,8 @@ class Pycos(pycos.Pycos, metaclass=Singleton):
 
             location = Location(*(tcp_sock.getsockname()[0:2]))
             addrinfo.ip = location.addr
-            if ext_ip_addr:
-                location.addr = ext_ip_addr
+            if ext_host:
+                location.addr = ext_host
 
             tcp_sock.listen(32)
             logger.info('TCP server "%s" @ %s', name if name else '', location)
@@ -650,7 +653,6 @@ class Pycos(pycos.Pycos, metaclass=Singleton):
                 else:
                     self.broadcast = broadcast
                 self.netmask = netmask
-                self.ext_ip_addr = None
                 if os.name == 'nt':
                     self.bind_addr = ip
                 elif platform.system() in ('Darwin', 'DragonFlyBSD', 'FreeBSD', 'OpenBSD', 'NetBSD'):
@@ -660,6 +662,9 @@ class Pycos(pycos.Pycos, metaclass=Singleton):
                         self.bind_addr = self.broadcast
                 else:
                     self.bind_addr = self.broadcast
+                self.location = None
+                self.tcp_sock = None
+                self.udp_sock = None
 
         if socket_family not in (None, socket.AF_INET, socket.AF_INET6):
             return None
@@ -749,56 +754,81 @@ class Pycos(pycos.Pycos, metaclass=Singleton):
         else:
             if not host:
                 host = socket.gethostname()
-            netmask = None
-            for sock_family in socket_families:
-                try:
-                    addrs = socket.getaddrinfo(host, None, sock_family, socket.SOCK_STREAM)
-                except Exception:
-                    continue
-                for addr in addrs:
-                    ifn = addr[-1][-1]
-                    if sock_family == socket.AF_INET:
-                        broadcast = '<broadcast>'
-                        addr = addr[-1][0]
-                    else:  # sock_family == socket.AF_INET6
-                        addr = Pycos.host_ipaddr(addr[-1][0])
-                        broadcast = pycos.config.IPV6_MULTICAST_GROUP
-                        logger.warning('IPv6 may not work without "netifaces" package!')
-                    addrinfo = AddrInfo(sock_family, addr, ifn, broadcast, netmask)
-                    if hosts:
-                        if addrinfo.ip in hosts:
-                            return addrinfo
-                        else:
-                            continue
-                    addrinfos.append(addrinfo)
+            addrs = socket.getaddrinfo(host, None)
+            for addr in addrs:
+                ifn = addr[-1][-1]
+                sock_family = addr[0]
+                if sock_family == socket.AF_INET:
+                    broadcast = '<broadcast>'
+                    addr = addr[-1][0]
+                else:  # sock_family == socket.AF_INET6
+                    addr = Pycos.host_ipaddr(addr[-1][0])
+                    broadcast = pycos.config.IPV6_MULTICAST_GROUP
+                    logger.warning('IPv6 may not work without "netifaces" package!')
+                addrinfo = AddrInfo(sock_family, addr, ifn, broadcast, None)
+                if hosts:
+                    if addrinfo.ip in hosts:
+                        return addrinfo
+                    else:
+                        continue
+                addrinfos.append(addrinfo)
 
-        best = None
-        for sock_family in socket_families:
-            for addrinfo in addrinfos:
-                if addrinfo.ip in hosts:
-                    return addrinfo
-                if addrinfo.family != sock_family:
-                    continue
-                if addrinfo.family == socket.AF_INET:
-                    if not best or (len(best.ip) < len(addrinfo.ip)) or best.ip.startswith('127'):
-                        best = addrinfo
-                else:
-                    if addrinfo.ip.startswith('fd'):
-                        # TODO: How to detect / avoid temporary addresses (privacy extensions)?
-                        if addrinfo.ifn:
-                            return addrinfo
-                    if not best or (len(best.ip) < len(addrinfo.ip)) or best.ip.startswith('fe80:'):
-                        best = addrinfo
-            if best and best.family == socket.AF_INET and (not best.ip.startswith('127')):
-                return best
-        return best
+        best = {}
+        for addrinfo in addrinfos:
+            if addrinfo.ip in hosts:
+                return addrinfo
+            cur = best.get(addrinfo.family, None)
+            if not cur:
+                best[addrinfo.family] = addrinfo
+                continue
+            if ((addrinfo.family == socket.AF_INET6 and addrinfo.ip.startswith('fd')) or
+                (addrinfo.family == socket.AF_INET)):
+                cur = best.get(addrinfo.family, None)
+                if (cur.ip.startswith('fe80:') or cur.ip == '::1' or
+                    cur.ip.startswith('127.') or (not cur.ifn and addrinfo.ifn) or
+                    len(addrinfo.ip) > len(cur.ip)):
+                    best[addrinfo.family] = addrinfo
+        addrinfo = best.get(socket_families[0], None)
+        if addrinfo:
+            if ((addrinfo.family == socket.AF_INET and not addrinfo.ip.startswith('127.')) or
+                (addrinfo.family == socket.AF_INET6 and (not addrinfo.ip.startswith('::1') and
+                                                         not addrinfo.ip.startswith('fe80:')))):
+                return addrinfo
+        elif len(socket_families) >= 1:
+            return best.get(socket_families[1], None)
+        return None
 
-    # TODO: is there a better approach to find best suitable address to select
-    # (with netmask)?
     def _ip_addrinfo_(self, ip):
         """
         Internal use only.
         """
+        if ipaddress:
+            ip_addr = ipaddress.ip_address(ip)
+            for addrinfo in self._addrinfos:
+                if not addrinfo.netmask:
+                    continue
+                if isinstance(ip_addr, ipaddress.IPv4Address):
+                    if addrinfo.family != socket.AF_INET:
+                        continue
+                    try:
+                        net = ipaddress.IPv4Network((addrinfo.ip, addrinfo.netmask), strict=False)
+                        if ipaddress.IPv4Address(ip) in net:
+                            return addrinfo
+                    except Exception:
+                        pass
+                elif isinstance(ip_addr, ipaddress.IPv6Address):
+                    if addrinfo.family != socket.AF_INET6:
+                        continue
+                    try:
+                        # IPv6Network works with prefixlen, but not netmask, so try common
+                        # prefixlen of 64 and check if netmask is same
+                        net = ipaddress.IPv6Network(addrinfo.ip + '/64', strict=False)
+                        if (net.netmask == ipaddress.IPv6Address(addrinfo.netmask) and
+                            ipaddress.IPv6Address(ip) in net):
+                            return addrinfo
+                    except Exception:
+                        pass
+
         best = (0, self._addrinfos[0])
         for addrinfo in self._addrinfos:
             i, n = 0, min(len(ip), len(addrinfo.ip))
